//
// russellville test suite menu v1.0
//
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <curses.h>
#include <panel.h>
#include <sys/utsname.h>

// defines
#define KEY_ESC 27

// prototypes
void menu_init(char *);
void display_init(void);
void display_close(void);
static void display_update(void);

// things for curses
int		G_input;
int		G_loop_flag = 1;
unsigned long G_ticks = 0;
static	WINDOW *G_screen;
static	WINDOW *G_box;

// things for system info
struct	utsname G_uts;

// things for time display
int		G_nowsec, G_nowmin, G_nowhour, G_nowday, G_nowwday, G_nowmonth, G_nowyear;
time_t	G_now;
struct	tm *G_nowtm;

// for program config
int		G_menu_count;
char	G_conf_file[] = "menu.conf";
char 	G_menu_title[26][128];
char	G_menu_command[26][128];
char	G_line[] = "______________________________________________________________________________";

/////////////////////////////////////
void menu_init(char *menu_file_name)
{
FILE *fp;
int len;
char stmp[128];
int is_command = 0;

	G_menu_count = -1;
	fp = fopen(menu_file_name,"r");

	if(fp != NULL) {
		G_menu_count++;
		while(1) {

			fgets(stmp,128,fp);						// read a line
			if(feof(fp)) break;						// EOF? we're done!
			len = strlen(stmp); 					// get it's length

			if(len == 0) continue;					// empty line? skip it!
			if(stmp[0] == '#') continue;			// comment? skip it!

			if(is_command) {						// it's a command, prep and snarf it
				stmp[len-1] = '\0';					// trim the newline from the end
				strncpy(G_menu_command[G_menu_count],stmp,128);	// snarf it!
				is_command = 0;						// reset the flag
				if(++G_menu_count > 25) break;		// bump the counter
			} else {								// else it's a menu string
				stmp[len-1] = '\0';					// strip the newline
				stmp[34] = '\0';					// trim the length in case it's too long
				strncpy(G_menu_title[G_menu_count],stmp,128);	// snarf it!
				is_command = 1;						// set the flag
			}
		}
		fclose(fp);
	}
	return;
}

/////////////////////////////////////
void display_init(void) {

	G_screen = initscr();
/*
	if(has_colors() == FALSE) {
		printf("has_colors failed.\n");
	} else {
		printf("has_colors OK!\n");
	}

	if(can_change_color() == FALSE) {
		printf("can_change_color failed.\n");
	} else {
		printf("can_change_color OK!\n");
	}
*/
	noecho();
	cbreak();
	nodelay(G_screen, TRUE);

	refresh();
	wrefresh(G_screen);

	G_box = newwin(24, 80, 1, 1);
	box(G_box, ACS_VLINE, ACS_HLINE);

	return;
}

/////////////////////////////////////
void display_close(void) {
   endwin();
}

/////////////////////////////////////
static void display_update(void) {
int i;

	curs_set(0);
	
	mvwprintw(G_box,0,2," russellville test suite v1.0 ");
	mvwprintw(G_box,1,2,"  ticks: %08ld",G_ticks);

	mvwprintw(G_box,2,2,"systime: %02d:%02d:%02d %02d/%02d/%04d", 
	  G_nowhour, G_nowmin, G_nowsec, G_nowmonth, G_nowday, G_nowyear);

	mvwprintw(G_box,3,2," kernel: %s",G_uts.release);
	mvwprintw(G_box,4,2,"version: %s",G_uts.version);

	mvwprintw(G_box,7,1,G_line);

	for(i=0;i<13;i++) {
		mvwprintw(G_box, 9+i, 2,"%c) %s",'a'+i,G_menu_title[i]);
	}

	for(i=0;i<13;i++) {
		mvwprintw(G_box, 9+i,41,"%c) %s",'n'+i,G_menu_title[i+13]);
	}

	mvwprintw(G_box,22,1,G_line);
	mvwprintw(G_box,1,55,"|");
	mvwprintw(G_box,2,55,"|");
	mvwprintw(G_box,3,55,"|");
	mvwprintw(G_box,4,55,"|");
	mvwprintw(G_box,5,55,"|");
	mvwprintw(G_box,6,55,"|");
	mvwprintw(G_box,23,65,"(esc to exit)");

	wrefresh(G_box);
	refresh();
}

void get_time(void) {
	// Get the current date/time
	G_now = time (NULL);
	G_nowtm = localtime (&G_now);
	G_nowsec = G_nowtm->tm_sec;
	G_nowmin = G_nowtm->tm_min;
	G_nowhour = G_nowtm->tm_hour;
	G_nowday = G_nowtm->tm_mday;
	G_nowwday = G_nowtm->tm_wday;
	G_nowmonth = G_nowtm->tm_mon + 1;
	G_nowyear = G_nowtm->tm_year + 1900;
}

int main(void) {
int key_value;
FILE *p;
char stmp[256];

	// init menu
	menu_init(G_conf_file);

	if(G_menu_count < 0) {
		fprintf(stderr,"can't open config file (%s)!\n",G_conf_file);
		exit(0);
	}

	// init curses
	display_init();

	// get system info
	uname(&G_uts);

	while(G_loop_flag) {

		G_input = getch();
		//key_value = atoi(G_input);
		switch(G_input) {
			case KEY_UP:
			case KEY_DOWN:
			case KEY_LEFT:
			case KEY_RIGHT:
			break;

			case KEY_ESC:
				G_loop_flag = 0;
			break;

			// case '1':
				// system("DISPLAY=\":0.1\" gnome-terminal --command \"top\" & ");
			// break;
			// case '2':
				// system("DISPLAY=\":0.0\" gnome-terminal --command \"top\" & ");
			// break;
			// case '3':
			// break;

			default:
				menu_init(G_conf_file);
				if(islower(G_input)) {
					//printf("****%s****",G_menu_command[G_input-'a']);
					system(G_menu_command[G_input-'a']);
					// p = popen(G_menu_command[G_input - 'a'],"r");
					// while(1) {
						// fgets(stmp,256,p);
						// if(feof(p)) break;
						// printf("**** %s *****\n",stmp);
						// while(1){};
					// }
					// close(p);
				}
			break;
		}

		get_time();
		display_update();
		usleep(100000);

		G_ticks++;
	}
	display_close();

	// printf("TEST ENDS\n"); 
	return 0;	 		   						                                                          
}

// the end
